package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.ArrayList;
import java.util.Vector;
import java.io.File;

import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.DynamicListModel;
import org.greenstone.gatherer.metadata.MetadataSet;
import org.greenstone.gatherer.metadata.MetadataSetManager;

public class MetadataSetDialog 
    extends ModalDialog {

    static private Dimension SIZE = new Dimension(600, 300);
    static private Dimension ADD_SIZE = new Dimension(600, 500);
    
    private ArrayList current_metadata_sets;
    private DynamicListModel current_metadata_model;

    private JButton add_button = null;
    private JButton edit_button = null;
    private JButton remove_button = null;

    private JButton close_button = null;
    
    private JList current_set_list = null;
    private MetadataSetDialog set_dialog = null;
    private boolean sets_changed = false;
    
    public MetadataSetDialog() {
	super(Gatherer.g_man, true);
	set_dialog = this;
	
	setJMenuBar(new SimpleMenuBar("choosingmetadatasets"));
	setSize(SIZE);
	setTitle(Dictionary.get("MetadataSetDialog.Title"));
	
	current_metadata_sets = MetadataSetManager.getMetadataSets();
	current_metadata_model = new DynamicListModel();
	
	int current_size = current_metadata_sets.size();
	for (int i=0; i<current_size; i++) {
	    current_metadata_model.addElement(current_metadata_sets.get(i));
	}
	JPanel content_pane = (JPanel) getContentPane();
        content_pane.setOpaque(true);
	
	JLabel current_metadata_sets_label = new JLabel(Dictionary.get("MetadataSetDialog.Current_Sets"));
	current_metadata_sets_label.setOpaque(true);
	current_set_list = new JList(current_metadata_model);
	current_set_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	JPanel button_pane = new JPanel();
	add_button = new GLIButton(Dictionary.get("MetadataSetDialog.Add"), Dictionary.get("MetadataSetDialog.Add_Tooltip"));
	add_button.setEnabled(true);
	
	edit_button = new GLIButton(Dictionary.get("MetadataSetDialog.Edit"), Dictionary.get("MetadataSetDialog.Edit_Tooltip"));
	edit_button.setEnabled(false);
	
	remove_button = new GLIButton(Dictionary.get("MetadataSetDialog.Remove"), Dictionary.get("MetadataSetDialog.Remove_Tooltip"));
	remove_button.setEnabled(false);
	
	close_button = new GLIButton(Dictionary.get("General.Close"), Dictionary.get("General.Close_Tooltip"));
	close_button.setEnabled(true);


	// Add listeners
	add_button.addActionListener(new AddButtonListener());
	edit_button.addActionListener(new EditButtonListener());
	remove_button.addActionListener(new RemoveButtonListener());
	close_button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    set_dialog.dispose();
		}
	    });
	current_set_list.addListSelectionListener(new MetadataSetListSelectionListener());

	button_pane.setLayout(new GridLayout(2,3));
	button_pane.add(add_button);
	button_pane.add(edit_button);
	button_pane.add(remove_button);
	button_pane.add(new JPanel());
	button_pane.add(new JPanel());
	button_pane.add(close_button);

	content_pane.setLayout(new BorderLayout());
	content_pane.add(current_metadata_sets_label, BorderLayout.NORTH);
	content_pane.add(new JScrollPane(current_set_list), BorderLayout.CENTER);
	content_pane.add(button_pane, BorderLayout.SOUTH);
	
	// Show
	Dimension screen_size = Configuration.screen_size;
	setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
	setVisible(true);

    }

    public boolean setsChanged() {
	return sets_changed;
    }
    
    private class AddButtonListener
	implements ActionListener {

	public void actionPerformed(ActionEvent event) {
	    AddMetadataSetPrompt amsp = new AddMetadataSetPrompt();
	    if (!amsp.isCancelled()) {
		sets_changed = true;
	    }
	}
    }
    
    private class EditButtonListener
	implements ActionListener {
	
	public void actionPerformed(ActionEvent event) {
	    // do a pop up message
	    JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("MetadataSetDialog.Edit_Message"), Dictionary.get("MetadataSetDialog.Edit_Message_Title"), JOptionPane.INFORMATION_MESSAGE);
	}
    }

    private class RemoveButtonListener
	implements ActionListener {
	
	public void actionPerformed(ActionEvent event) {
	    MetadataSet metadata_set = (MetadataSet) current_set_list.getSelectedValue();
	    Gatherer.c_man.removeMetadataSet(metadata_set);
	    current_metadata_model.removeElement(metadata_set);
	    sets_changed = true;
	    
	}

    }


    private class MetadataSetListSelectionListener
	implements ListSelectionListener {
	
	public void valueChanged(ListSelectionEvent event)
	{
	    // Wait until we get a stable event
	    if (event.getValueIsAdjusting()) {
		return;
	    }

	    // Now we can process it
	    if (!current_set_list.isSelectionEmpty() && !((MetadataSet)current_set_list.getSelectedValue()).getNamespace().equals(MetadataSetManager.EXTRACTED_METADATA_NAMESPACE)) {
		remove_button.setEnabled(true);
		edit_button.setEnabled(true);
	    }
	    else { 
		remove_button.setEnabled(false);
		edit_button.setEnabled(false);
	    }
	    
	}
	
    }

    private class AddMetadataSetPrompt
	extends ModalDialog {
	
	private JDialog add_set_dialog;
	private boolean cancelled = false;
	
	private JList elements_list;
	private JList available_sets_list;

	private JButton add_button;

	public AddMetadataSetPrompt() {
	    super(Gatherer.g_man, true);
	    add_set_dialog = this;

	    setModal(true);
	    setJMenuBar(new SimpleMenuBar("choosingmetadatasets"));
	    setSize(ADD_SIZE);
	    setTitle(Dictionary.get("MetadataSetDialog.Add_Title"));

	    // Show the metadata sets (except extracted, exploded, and currently assigned sets) in the GLI metadata folder
	    ArrayList all_metadata_sets = MetadataSetManager.listMetadataSets(new File(Gatherer.getGLIMetadataDirectoryPath()));
	    ArrayList current_metadata_sets = MetadataSetManager.getMetadataSets();
	    DynamicListModel valid_sets_model = new DynamicListModel();
	    for (int i=0; i<all_metadata_sets.size(); i++) {
		MetadataSet set = (MetadataSet) all_metadata_sets.get(i);
		if (set.getNamespace().equals(MetadataSetManager.EXTRACTED_METADATA_NAMESPACE) || set.getNamespace().equals(MetadataSetManager.EXPLODED_METADATA_NAMESPACE)) {
		    continue;
		}
		// indexOf uses the equals() method, which for MetadataSets compares the toString() output, not the objects
		if (current_metadata_sets.indexOf(set)!=-1) {
		    continue;
		}
		valid_sets_model.addElement(set);
	    }

	    JPanel center_pane = new JPanel();
	    JPanel sets_pane = new JPanel();
	    JLabel sets_label = new JLabel(Dictionary.get("MetadataSetDialog.Available_Sets"));
	    sets_label.setOpaque(false);
	    
	    available_sets_list = new JList(valid_sets_model);
	    available_sets_list.addListSelectionListener(new AvailableSetListSelectionListener());
	    available_sets_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    JPanel elements_pane = new JPanel();
	    JLabel elements_label = new JLabel(Dictionary.get("MetadataSetDialog.Elements"));
	    
	    elements_list = new JList();
	    elements_list.setCellRenderer(new MetadataElementListCellRenderer());
	    elements_list.setBackground(Configuration.getColor("coloring.collection_tree_background", false));
	    elements_list.setForeground(Configuration.getColor("coloring.collection_tree_foreground", false));
	    elements_list.setSelectionBackground(Configuration.getColor("coloring.collection_tree_background", false));
	    elements_list.setSelectionForeground(Configuration.getColor("coloring.collection_tree_foreground", false));
	    
	    JPanel button_pane = new JPanel();
	    add_button = new GLIButton(Dictionary.get("MetadataSetDialog.Add_Set"), Dictionary.get("MetadataSetDialog.Add_Set_Tooltip"));
	    add_button.setEnabled(false);
	    JButton browse_button = new GLIButton(Dictionary.get("MetadataSetDialog.Browse"), Dictionary.get("MetadataSetDialog.Browse_Tooltip"));
	    browse_button.setEnabled(true);
	    JButton cancel_button = new GLIButton(Dictionary.get("General.Cancel"), Dictionary.get("General.Pure_Cancel_Tooltip"));
	    
	    add_button.addActionListener(new AddSetActionListener());
	    browse_button.addActionListener(new BrowseActionListener());
	    cancel_button.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent event) {
			cancelled = true;
			add_set_dialog.dispose();
		    }
		});
	    
	    // Layout
	    sets_pane.setLayout(new BorderLayout());
	    sets_pane.add(sets_label, BorderLayout.NORTH);
	    sets_pane.add(new JScrollPane(available_sets_list), BorderLayout.CENTER);

	    elements_pane.setLayout(new BorderLayout());
	    elements_pane.add(elements_label, BorderLayout.NORTH);
	    elements_pane.add(new JScrollPane(elements_list), BorderLayout.CENTER);

	    center_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    center_pane.setLayout(new GridLayout(2,1,0,5));
	    center_pane.add(sets_pane);
	    center_pane.add(elements_pane);
	    
	    button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    button_pane.setLayout(new GridLayout(1,3,5,0));
	    button_pane.add(add_button);
	    button_pane.add(browse_button);
	    button_pane.add(cancel_button);
	    
	    JPanel content_pane = (JPanel) getContentPane();
	    content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    content_pane.setLayout(new BorderLayout());
	    content_pane.add(center_pane, BorderLayout.CENTER);
	    content_pane.add(button_pane, BorderLayout.SOUTH);

	    // Show
	    Dimension screen_size = Configuration.screen_size;
	    setLocation((screen_size.width - ADD_SIZE.width) / 2, (screen_size.height - ADD_SIZE.height) / 2);
	    setVisible(true);
	    

	    
	}

	public boolean isCancelled() {
	    return cancelled;
	}

	private class AddSetActionListener
	    implements ActionListener {

	    public void actionPerformed(ActionEvent event) {
		if (available_sets_list.isSelectionEmpty()) {
		    return;
		}
		MetadataSet metadata_set = (MetadataSet) available_sets_list.getSelectedValue();
		Gatherer.c_man.importMetadataSet(metadata_set);
		current_metadata_model.addElement(metadata_set);
		cancelled = false;
		add_set_dialog.dispose();

	    }
	}

	private class BrowseActionListener
	    implements ActionListener {

	    public void actionPerformed(ActionEvent event) {
		JFileChooser chooser = new JFileChooser(new File(Gatherer.getGLIMetadataDirectoryPath()));
		chooser.setFileFilter(new MetadataSet.MetadataSetFileFilter());
		chooser.setDialogTitle(Dictionary.get("MetadataSetDialog.Add_Title"));
		int return_val = chooser.showDialog(Gatherer.g_man, Dictionary.get("MetadataSetDialog.Add_Set"));
		if (return_val == JFileChooser.APPROVE_OPTION) {
		    MetadataSet meta_set = new MetadataSet(chooser.getSelectedFile());
		    Gatherer.c_man.importMetadataSet(meta_set);
		    current_metadata_model.addElement(meta_set);
		    cancelled = false;
		    add_set_dialog.dispose();
		} else {
		    // we do nothing - user may want to add from the other dialog
		}
	    }
	}
	
		
	private class AvailableSetListSelectionListener 
	    implements ListSelectionListener
	{
	    public void valueChanged(ListSelectionEvent event)
	    {
		// Wait until we get a stable event
		if (event.getValueIsAdjusting()) {
		    return;
		}

		if (!available_sets_list.isSelectionEmpty()) {
		    // Retrieve the selected set
		    MetadataSet metadata_set = (MetadataSet) available_sets_list.getSelectedValue();
		    elements_list.setListData(new Vector(metadata_set.getMetadataSetElements()));
		    add_button.setEnabled(true);
		}
		else {
		    elements_list.setListData(new String[0]);
		    add_button.setEnabled(false);
		}
	    }
	}
	
	
    }
}
